package com.perforce.bumble.pending;

import java.util.ArrayList;
import java.util.List;

import com.perforce.bumble.connection.ConnectionService;
import com.perforce.bumble.connection.DefaultService;
import com.perforce.p4java.core.IChangelist;
import com.perforce.p4java.core.file.FileSpecBuilder;
import com.perforce.p4java.core.file.FileSpecOpStatus;
import com.perforce.p4java.core.file.IFileSpec;
import com.perforce.p4java.exception.P4JavaException;
import com.perforce.p4java.impl.generic.core.Changelist;
import com.perforce.p4java.option.changelist.SubmitOptions;
import com.perforce.p4java.option.client.ReopenFilesOptions;
import com.perforce.p4java.option.server.OpenedFilesOptions;
import com.perforce.p4java.server.IOptionsServer;

public class PendingService extends DefaultService {

	public PendingService(IOptionsServer p4) throws P4JavaException {
		super(p4);
	}

	public List<Pending> getFiles() throws P4JavaException {

		OpenedFilesOptions openOps = new OpenedFilesOptions();
		List<IFileSpec> open = iclient.openedFiles(spec, openOps);

		List<Pending> list = new ArrayList<>();
		for (IFileSpec file : open) {
			if (file != null && file.getAction() != null) {
				String local = toLocal(file);
				Pending pending = new Pending(local, file);
				list.add(pending);
			}
		}
		return list;
	}

	public long submit(String desc, String[] files) throws P4JavaException {

		// create new pending change and add description
		IChangelist change = new Changelist();
		change.setDescription(desc);
		change = iclient.createChangelist(change);

		// move files to pending change
		ReopenFilesOptions reopenOpts = new ReopenFilesOptions();
		reopenOpts.setChangelistId(change.getId());

		List<String> fileList = new ArrayList<>();
		for (String f : files) {
			String base = iclient.getRoot() + "/";
			fileList.add(base + f);
		}
		
		List<IFileSpec> fileSpec = FileSpecBuilder.makeFileSpecList(fileList);
		List<IFileSpec> status = iclient.reopenFiles(fileSpec, reopenOpts);
		ConnectionService.validate(status);

		// submit files
		SubmitOptions submitOpts = new SubmitOptions();
		List<IFileSpec> submitted = change.submit(submitOpts);
		ConnectionService.validate(submitted, "Submitted as change");

		long cngNumber = findSubmittedChange(submitted);
		return cngNumber;
	}

	private long findSubmittedChange(List<IFileSpec> submitted) {
		long change = 0;
		for (IFileSpec spec : submitted) {
			if (spec.getOpStatus() != FileSpecOpStatus.VALID) {
				String msg = spec.getStatusMessage();
				String cng = "Submitted as change ";
				if (msg.startsWith(cng)) {
					try {
						String id = msg.substring(cng.length());
						change = Long.parseLong(id);
					} catch (NumberFormatException e) {
						change = -1;
					}
				}
			}
		}
		return change;
	}
}
